<?php
class Sabai_Addon_Questions extends Sabai_Addon
    implements Sabai_Addon_Field_ITypes,
               Sabai_Addon_Taxonomy_ITaxonomies,
               Sabai_Addon_System_IMainRouter,
               Sabai_Addon_System_IAdminRouter,
               Sabai_Addon_Content_IContentTypes,
               Sabai_Addon_Widgets_IWidgets,
               Sabai_Addon_System_IUserMenus,
               Sabai_Addon_System_IAdminMenus,
               Sabai_Addon_System_IMainMenus
{
    const VERSION = '1.2.31', PACKAGE = 'sabai-discuss';
    
    protected $_path, $_allowedAccess, $_questionsBundleName, $_answersBundleName, $_tagsBundleName, $_categoriesBundleName;
    
    protected function _init()
    {
        if (!isset($this->_config['questions_name'])) $this->_config['questions_name'] = $this->getQuestionsSlug();
        $this->_path = $this->_application->Path(dirname(__FILE__) . '/Questions');
        $this->_questionsBundleName = $this->_config['questions_name'];
        $this->_answersBundleName = $this->_config['questions_name'] . '_answers';
        $this->_tagsBundleName = $this->_config['questions_name'] . '_tags';
        $this->_categoriesBundleName = $this->_config['questions_name'] . '_categories';
        
        return $this;
    }
    
    public function isCloneable()
    {
        return !$this->hasParent();
    }
    
    public function getQuestionsBundleName()
    {
        return $this->_questionsBundleName;
    }
    
    public function getAnswersBundleName()
    {
        return $this->_answersBundleName;
    }
            
    public function getCategoriesBundleName()
    {
        return $this->_categoriesBundleName;
    }
        
    public function getTagsBundleName()
    {
        return $this->_tagsBundleName;
    }
        
    public function getQuestionsSlug()
    {
        return $this->getSlug('questions');
    }
        
    public function getSlug($name)
    {
        return isset($this->_config['slug'][$name]) // check for backward compat with 1.0.x versions
            ? $this->_config['slug'][$name]
            : $name;
    }
    
    public function getPageTitle($name = 'main')
    {
        return isset($this->_config['page_title'][$name]) // check for backward compat with 1.0.x versions
            ? $this->_config['page_title'][$name]
            : $this->_config['title'];
    }
    
    /* Start implementation of Sabai_Addon_Field_ITypes */

    public function fieldGetTypeNames()
    {
        return array('questions_resolved', 'questions_closed', 'questions_answer_accepted');
    }

    public function fieldGetType($name)
    {
        require_once $this->_path . '/FieldType.php';
        return new Sabai_Addon_Questions_FieldType($this, $name);
    }

    /* End implementation of Sabai_Addon_Field_ITypes */
    
    /* Start implementation of Sabai_Addon_System_IMainRouter */
    
    public function systemGetMainRoutes()
    {        
        $routes = array(
            '/' . $this->getQuestionsSlug() => array(
                'controller' => 'Questions',
                'access_callback' => true,
                'callback_addon' => 'Content',
                'callback_path' => 'posts',
                'title_callback' => true,
                'type' => Sabai::ROUTE_TAB,
                'data' => array(
                    'bundle_name' => $this->_config['questions_name'],
                ),
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('ask') => array(
                'controller' => 'AskQuestion',
                'access_callback' => true,
                'title_callback' => true,
                'callback_path' => 'ask',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/add' => array(
                'forward' => '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('ask'),
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/feed' => array(
                'controller' => 'Feed',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/users/:user_name' => array(
                'controller' => 'UserQuestions',
                'access_callback' => true,
                'title_callback' => true,
                'type' => Sabai::ROUTE_TAB,
                'weight' => 1,
                'controller_addon' => 'Questions',
                'priority' => 5,
                'callback_path' => 'user_questions',
                'data' => array('clear_tabs' => true),
                'format' => array(':user_name' => '.+'),
            ),
            '/' . $this->getQuestionsSlug() . '/users/:user_name/' . $this->getSlug('answers') => array(
                'controller' => 'UserAnswers',
                'access_callback' => true,
                'type' => Sabai::ROUTE_TAB,
                'title_callback' => true,
                'weight' => 2,
                'callback_path' => 'user_answers',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/users/:user_name/favorites' => array(
                'controller' => 'UserFavorites',
                'type' => Sabai::ROUTE_TAB,
                'title_callback' => true,
                'callback_path' => 'user_favorites',
                'weight' => 3,
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/users/:user_name/feed' => array(
                'controller' => 'UserFeed',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('question') . '/:slug/close' => array(
                'controller' => 'CloseQuestion',
                'access_callback' => true,
                'callback_path' => 'close',
                'title_callback' => true,
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('question') . '/:slug/' . $this->getSlug('answers') => array(
                'controller' => 'QuestionAnswers',
                'access_callback' => true,
                'title_callback' => true,
                'type' => Sabai::ROUTE_INLINE_TAB,
                'callback_path' => 'question_answers',
                'controller_addon' => 'Questions',
                'priority' => 5,
                'weight' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('question') . '/:slug/' . $this->getSlug('answers') . '/add' => array(
                'controller' => 'AddAnswer',
                'access_callback' => true,
                'title_callback' => true,
                'callback_path' => 'answer',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('categories') => array(
                'controller' => 'ListHierarchicalTerms',
                'access_callback' => true,
                'title_callback' => true,
                'callback_path' => 'categories',
                'controller_addon' => 'Taxonomy',
                'priority' => 5,
                'type' => Sabai::ROUTE_TAB,
                'weight' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('categories') . '/:slug/' . $this->getQuestionsSlug() => array(
                'controller' => 'TermQuestions',
                'title_callback' => true,
                'type' => Sabai::ROUTE_INLINE_TAB,
                'weight' => 1,
                'controller_addon' => 'Questions',
                'callback_path' => 'term_questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('tags') => array(
                'controller' => 'Tags',
                'title_callback' => true,
                'access_callback' => true,
                'callback_path' => 'tags',
                'controller_addon' => 'Questions',
                'priority' => 5,
                'type' => Sabai::ROUTE_TAB,
                'weight' => 10,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('tags') . '/:slug/' . $this->getQuestionsSlug() => array(
                'controller' => 'TermQuestions',
                'title_callback' => true,
                'type' => Sabai::ROUTE_INLINE_TAB,
                'weight' => 1,
                'controller_addon' => 'Questions',
                'callback_path' => 'term_questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers') => array(
                'controller' => 'Answers',
                'access_callback' => true,
                'title_callback' => true,
                'callback_path' => 'answers',
                'controller_addon' => 'Questions',
                'priority' => 5,
                'data' => array('clear_tabs' => true),
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers') . '/:entity_id' => array(
                'controller' => 'RedirectToQuestion',
                'format' => array(':entity_id' => '\d+'),
                'access_callback' => true,
                'title_callback' => true,
                'callback_addon' => 'Content',
                'callback_path' => 'child_post',
                'priority' => 5,
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers') . '/:entity_id/accept' => array(
                'controller' => 'AcceptAnswer',
                'title_callback' => true,
                'type' => Sabai::ROUTE_CALLBACK,
                'priority' => 5,
                'callback_path' => 'accept',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/flags' => array(
                'controller' => 'FlaggedPosts',
                'title_callback' => true,
                'callback_path' => 'flags',
                'controller_addon' => 'Questions',
                'priority' => 5,
            ),
            '/' . $this->getQuestionsSlug() . '/my' => array(
                'forward' => '/' . $this->getQuestionsSlug() . '/users/:user_name',
                'callback_path' => 'my',
                'access_callback' => true,
            ),
            '/' . $this->getQuestionsSlug() . '/my/:my_content' => array(
                'forward' => '/' . $this->getQuestionsSlug() . '/users/:user_name/:my_content',
                'callback_path' => 'my',
                'access_callback' => true,
            ),
        );
        if (!$this->hasParent()) {
            $routes += array(
                '/sabai/questions' => array(
                    'controller' => 'AllQuestions',
                    'type' => Sabai::ROUTE_CALLBACK,
                    'access_callback' => true,
                    'callback_path' => 'questions',
                    'controller_addon' => 'Questions',
                    'priority' => 5,
                ),
            );
        }
        
        return $routes;
    }

    public function systemOnAccessMainRoute(Sabai_Context $context, $path, $accessType, array &$route)
    {
        switch ($path) {
            case 'posts':
                if (!$this->_isAllowedAccess()) {
                    $context->setErrorUrl(rtrim($this->_application->getScriptUrl('main'), '/') . '/' . $this->_config['access']['redirect'])
                        ->setFlashEnabled(false); // do not show error message on next access
                    return false;
                }
                $context->config = $this->_config;
                return true;
            case 'ask':
                if ($accessType === Sabai::ROUTE_ACCESS_LINK) {
                    return true;
                }
                
                if ($this->_application->getUser()->isAnonymous()) {
                    if ($this->_application->HasPermission($context->bundle->name . '_add')) {
                        return true;
                    }
                    $path = false !== strpos($route['path'], ':slug') ? str_replace(':slug', $context->entity->getSlug(), $route['path']) : $route['path'];
                    $context->setUnauthorizedError($path);
                    return false;
                }
                return $this->_application->HasPermission($context->bundle->name . '_add');
            case 'close':
                if ($this->_application->HasPermission($this->_config['questions_name'] . '_manage')) {
                    return true;
                }
                if ($this->_application->HasPermission($this->_config['questions_name'] . '_close_any')) {
                    return true;
                }
                return $context->entity->getAuthorId() === $this->_application->getUser()->id
                    && $this->_application->HasPermission($this->_config['questions_name'] . '_close_own');
            case 'answer':
                if ($accessType === Sabai::ROUTE_ACCESS_LINK) {
                    return true;
                }
                if ($this->_application->getUser()->isAnonymous()) {
                    if ($this->_application->HasPermission($context->child_bundle->name . '_add')) {
                        return true;
                    }
                    $context->setUnauthorizedError($this->_application->Entity_Url($context->entity, '/answers/add'));
                    return false;
                }
                return $this->_application->HasPermission($context->child_bundle->name . '_add');
            case 'categories':
                if ($accessType !== Sabai::ROUTE_ACCESS_CONTENT) return !empty($this->_config['front']['categories_tab']);
                return (!$context->taxonomy_bundle = $this->_application->Entity_Bundle($this->_categoriesBundleName)) ? false : true;
            case 'tags':
                if ($accessType !== Sabai::ROUTE_ACCESS_CONTENT) return !empty($this->_config['front']['tags_tab']);
                return (!$context->taxonomy_bundle = $this->_application->Entity_Bundle($this->_tagsBundleName)) ? false : true;
            case 'user_questions':
                $user_name = $context->getRequest()->asStr('user_name');
                $context->identity = $this->_application->UserIdentityByUsername(rawurldecode($user_name));
                return !$context->identity->isAnonymous();
            case 'answers':
            case 'question_answers':
            case 'user_answers':
                if ($accessType !== Sabai::ROUTE_ACCESS_CONTENT) return true;
                return (!$context->child_bundle = $this->_application->Entity_Bundle($this->_answersBundleName)) ? false : true;
            case 'my':
                if ($this->_application->getUser()->isAnonymous()) {
                    return false;
                }
                $route['forward'] = str_replace(':user_name', rawurlencode($this->_application->getUser()->username), $route['forward']);
                return true;
            case 'questions':
                $context->addTemplateDir($this->_application->getPlatform()->getAssetsDir('sabai-discuss') . '/templates');
                return true;
        }
    }

    public function systemGetMainRouteTitle(Sabai_Context $context, $path, $title, $titleType, array $route)
    {
        switch ($path) {
            case 'posts':
            case 'term_questions':
                return $titleType === Sabai::ROUTE_TITLE_TAB || $titleType === Sabai::ROUTE_TITLE_TAB_DEFAULT
                    ? _x('Questions', 'tab', 'sabai-discuss')
                    : $this->getPageTitle();
            case 'ask':
                return __('Ask Question', 'sabai-discuss');
            case 'question_answers':
                return ($count = $context->entity->getSingleFieldValue('content_children_count', 'questions_answers'))
                    ? sprintf(__('Answers (%d)', 'sabai-discuss'), $count)
                    : __('Answers', 'sabai-discuss');
            case 'user_favorites':
                return __('Favorites', 'sabai-discuss');
            case 'categories':
                return !empty($this->_config['label']['categories']) ? $this->_config['label']['categories'] : __('Categories', 'sabai-discuss');
            case 'tags':
                return !empty($this->_config['label']['tags']) ? $this->_config['label']['tags'] : __('Tags', 'sabai-discuss');
            case 'flags':
                return __('Flags', 'sabai-discuss');
            case 'close':
                return __('Close Question', 'sabai-discuss');
            case 'user_questions':
                return $titleType === Sabai::ROUTE_TITLE_TAB_DEFAULT
                    ? __('Questions', 'sabai-discuss')
                    : sprintf(__('Posts by %s', 'sabai-discuss'), $context->identity->name);
            case 'user_answers':
                return __('Answers', 'sabai-discuss');
            case 'accept':
                return __('Accept Answer', 'sabai-discuss');
            case 'answer':
                return __('Post Answer', 'sabai-discuss');
            case 'answers':
                return __('All Answers', 'sabai-discuss');
        }
    }

    /* End implementation of Sabai_Addon_System_IMainRouter */
    
    /* Start implementation of Sabai_Addon_System_IAdminRouter */

    public function systemGetAdminRoutes()
    {
        return array(
            '/' . $this->getQuestionsSlug() . '/settings' => array(
                'controller' => 'Settings',
                'title_callback' => true,
                'controller_addon' => 'Questions',
                'access_callback' => true,
                'callback_path' => 'settings',
                'data' => array('clear_tabs' => true),
            ),
            '/' . $this->getQuestionsSlug() . '/settings/acl' => array(
                'controller' => 'AccessControl',
                'title_callback' => true,
                'type' => Sabai::ROUTE_TAB,
                'controller_addon' => 'Questions',
                'callback_path' => 'settings_acl',
            ),
            '/' . $this->getQuestionsSlug() . '/settings/emails' => array(
                'controller' => 'Emails',
                'title_callback' => true,
                'type' => Sabai::ROUTE_TAB,
                'controller_addon' => 'Questions',
                'callback_path' => 'settings_emails',
            ),
        );
    }

    public function systemOnAccessAdminRoute(Sabai_Context $context, $path, $accessType, array &$route)
    {
        switch ($path) {
            case 'settings':
                return true;
        }
    }

    public function systemGetAdminRouteTitle(Sabai_Context $context, $path, $title, $titleType, array $route)
    {
        switch ($path) {
            case 'settings':
                return $titleType === Sabai::ROUTE_TITLE_TAB_DEFAULT ? __('General', 'sabai-discuss') : sprintf(_x('%s Settings', 'Settings page title', 'sabai-discuss'), $this->_name);
            case 'settings_acl':
                return __('Access Control', 'sabai-discuss');
            case 'settings_emails':
                return __('Emails', 'sabai-discuss');
        }
    }

    /* End implementation of Sabai_Addon_System_IAdminRouter */
        
    /* Start implementation of Sabai_Addon_Content_IContentTypes */

    public function contentGetContentTypeNames()
    {
        return array($this->_questionsBundleName, $this->_answersBundleName);
    }

    public function contentGetContentType($name)
    {
        require_once $this->_path . '/ContentType.php';
        return new Sabai_Addon_Questions_ContentType($this, $name);
    }

    /* End implementation of Sabai_Addon_Content_IContentTypes */    
    
    public function getDefaultConfig()
    {
        return array(
            'front' => array(
                'perpage' => 20,
                'sort' => 'newest',
                'feature' => true,
                'categories_tab' => true,
                'tags_tab' => true,
                'buttons' => array(
                    'search' => 'sabai-btn-primary',
                    'ask' => 'sabai-btn-success',
                    'answer' => 'sabai-btn-success',
                ),
            ),
            'search' => array(
                'min_keyword_len' => 3,
            ),
            'spam' => array(
                'threshold' => 11,
                'auto_delete' => true,
                'delete_after' => 3,
            ),
            'reputation' => array(
                'points' => array(
                    'question_voted' => 5,
                    'question_voted_down' => -2,
                    'answer_voted' => 10,
                    'answer_voted_down' => -2,
                    'answer_accepted' => 15,
                    'answer_accepted_user' => 2,
                    'answer_vote_down' => -1,
                    'spam' => -100,
                    'question_unvoted' => -1,
                    'answer_unvoted' => -1,
                ),
            ),
            'access' => array(
                'type' => 'public',
                'roles' => array(),
                'redirect' => '',
            ),
            'rep_max' => 1000,
            'perm_rep_enable' => false,
            'perm_rep' => array(),
            'questions_name' => strtolower($this->_name),
            'page_title' => array(
                'main' => $this->_name,
                'category' => __('Category: %s', 'sabai-discuss'),
                'tag' => __('Tag: %s', 'sabai-discuss'),
            ),
            'label' => array(
                'categories' => __('Categories', 'sabai-discuss'),
                'category' => __('Category', 'sabai-discuss'),
                'tags' => __('Tags', 'sabai-discuss'),
                'tag' => __('Tag', 'sabai-discuss'),
            ),
            'slug' => array(
                'questions' => strtolower($this->_name),
                'answers' => 'answers',
                'ask' => 'ask',
                'question' => 'question',
                'tags' => 'tags',
                'categories' => 'categories',
            ),
        );
    }
    
    /* Start implmentation of Sabai_Addon_Widgets_IWidgets */
    
    public function widgetsGetWidgetNames()
    {
        if ($this->hasParent()) {
            return array();
        }
        return array('questions_recent', 'questions_popular', 'questions_unanswered', 'questions_categories', 'questions_tags', 'questions_search',
            'questions_answers_accepted', 'questions_askbtn', 'questions_featured', 'questions_related'
        );
    }
    
    public function widgetsGetWidget($name)
    {
        require_once $this->_path . '/Widget.php';
        return new Sabai_Addon_Questions_Widget($this, substr($name, strlen('questions_')));
    }
    
    /* End implmentation of Sabai_Addon_Widgets_IWidgets */
    
    /* Start implmentation of Sabai_Addon_System_IUserMenus */
    
    public function systemGetUserMenus()
    {
        if (!$this->_isAllowedAccess()) {
            return array();
        }
        $menus =  array(
            $this->_config['questions_name'] => array(
                'title' => $this->getPageTitle(),
                'url' => $this->_application->MainUrl('/' . $this->getQuestionsSlug()),
            ),
            $this->_config['questions_name'] . '_all' => array(
                'title' => __('All Questions', 'sabai-discuss'),
                'url' => $this->_application->MainUrl('/' . $this->getQuestionsSlug()),
                'parent' => $this->_config['questions_name'],
            ),
            $this->_config['questions_name'] . '_categories' => array(
                'title' => !empty($this->_config['label']['categories']) ? $this->_config['label']['categories'] : __('Categories', 'sabai-discuss'),
                'url' => $this->_application->MainUrl('/' . $this->getQuestionsSlug() . '/' . $this->getSlug('categories')),
                'parent' => $this->_config['questions_name'],
            ),
            $this->_config['questions_name'] . '_tags' => array(
                'title' => !empty($this->_config['label']['tags']) ? $this->_config['label']['tags'] : __('Tags', 'sabai-discuss'),
                'url' => $this->_application->MainUrl('/' . $this->getQuestionsSlug() . '/' . $this->getSlug('tags')),
                'parent' => $this->_config['questions_name'],
            ),
        );
        if (!$this->_application->HasPermission(array($this->_config['questions_name'] . '_manage', $this->_answersBundleName . '_manage'))) {
            return $menus;
        }
        
        $all_flag_count = $this->_application->Content_CountFlaggedPosts();
        $flag_count = 0;
        if (isset($all_flag_count[$this->_config['questions_name']])
            && $this->_application->HasPermission($this->_config['questions_name'] . '_manage')
        ) {
            $flag_count += $all_flag_count[$this->_config['questions_name']];
        }
        if (isset($all_flag_count[$this->_answersBundleName])
            && $this->_application->HasPermission($this->_answersBundleName . '_manage')
        ) {
            $flag_count += $all_flag_count[$this->_answersBundleName];
        }
        if ($flag_count > 0) {
            $menus[$this->_config['questions_name']]['title'] = sprintf(__('%s (%d)', 'sabai-discuss'), $this->getPageTitle(), $flag_count);
            $flags_title = sprintf(__('Flags (%d)', 'sabai-discuss'), $flag_count);
        } else {
            $flags_title = __('Flags', 'sabai-discuss');
        }
        $menus += array(
            $this->_config['questions_name'] . '_flags' => array(
                'title' => $flags_title,
                'url' => $this->_application->MainUrl('/' . $this->getQuestionsSlug() . '/flags'),
                'parent' => $this->_config['questions_name'],
            ),
        );
        
        return $menus;
    }
    
    public function systemGetUserProfileMenus(Sabai_UserIdentity $identity)
    {
        return array();
    }
    
    /* End implmentation of Sabai_Addon_System_IUserMenus */
    
    /* Start implmentation of Sabai_Addon_System_IAdminMenus */
    
    public function systemGetAdminMenus()
    {
        $icon_path = str_replace($this->_application->getPlatform()->getSiteUrl() . '/', '', $this->_application->getPlatform()->getAssetsUrl('sabai-discuss'));
        return array(
            '/' . $this->getQuestionsSlug() => array(
                'label' => $this->_name,
                'title' => __('Questions', 'sabai-discuss'),
                'icon' => $icon_path . '/images/icon.png',
                'icon_dark' => $icon_path . '/images/icon_dark.png',
            ),
            '/' . $this->getQuestionsSlug() . '/add' => array(
                'title' => __('Add Question', 'sabai-discuss'),
                'parent' => '/' . $this->getQuestionsSlug(),
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers') => array(
                'title' => __('Answers', 'sabai-discuss'),
                'parent' => '/' . $this->getQuestionsSlug(),
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('categories') => array(
                'title' => !empty($this->_config['label']['categories']) ? $this->_config['label']['categories'] : __('Categories', 'sabai-discuss'),
                'parent' => '/' . $this->getQuestionsSlug(),
            ),
            '/' . $this->getQuestionsSlug() . '/' . $this->getSlug('tags') => array(
                'title' => !empty($this->_config['label']['tags']) ? $this->_config['label']['tags'] : __('Tags', 'sabai-discuss'),
                'parent' => '/' . $this->getQuestionsSlug(),
            ),
            '/' . $this->getQuestionsSlug() . '/settings' => array(
                'title' => __('Settings', 'sabai-discuss'),
                'parent' => '/' . $this->getQuestionsSlug(),
                'weight' => 99,
            ),
        );
    }
    
    /* End implmentation of Sabai_Addon_System_IAdminMenus */
    
    /* Start implmentation of Sabai_Addon_System_IMainMenus */
    
    public function systemGetMainMenus()
    {
        return array(
            '/' . $this->getQuestionsSlug() => array(
                'title' => $this->getPageTitle(),
            ),
        );
    }
    
    /* End implmentation of Sabai_Addon_System_IMainMenus */
    
    /* Start implementation of Sabai_Addon_Taxonomy_ITaxonomies */

    public function taxonomyGetTaxonomyNames()
    {
        return array($this->_categoriesBundleName, $this->_tagsBundleName);
    }

    public function taxonomyGetTaxonomy($name)
    {
        require_once $this->_path . '/Taxonomy.php';
        return new Sabai_Addon_Questions_Taxonomy($this, $name);
    }

    /* End implementation of Sabai_Addon_Taxonomy_ITaxonomies */
    
    public function isUpgradeable($currentVersion, $newVersion)
    {
        if (!parent::isUpgradeable($currentVersion, $newVersion)) {
            return false;
        }
        if (version_compare($currentVersion, '1.1.0dev322', '<')) {
            $required_addons = array(
                'Content' => '1.1.0dev78',
                'Entity' => '1.1.0dev12',
                'Taxonomy' => '1.1.0',
                'Voting' => '1.0.2dev35'
            );
            return $this->_application->CheckAddonVersion($required_addons);
        }
        
        return true;
    }
            
    public function hasSettingsPage($currentVersion)
    {
        return '/' . $this->getQuestionsSlug() . '/settings';
    }
    
    public function isQuestionTrashable($question, $user)
    {        
        if (!$this->_application->Content_IsAuthor($question, $user)
            || !$this->_application->HasPermission($this->_config['questions_name'] . '_trash_own', $user->getIdentity())
        ) return false;
        
        // Questions that are resolved or have more than 1 answer are no longer deletable by the question owner 
        if ($question->getSingleFieldValue('questions_resolved')) return false;
        
        $answer_count = (int)$question->getSingleFieldValue('content_children_count', 'questions_answers');
        return $answer_count <= 1;
    }
    
    public function isAnswerTrashable($answer, $user)
    {        
        if (!$this->_application->Content_IsAuthor($answer, $user)
            || !$this->_application->HasPermission($this->_answersBundleName . '_trash_own', $user->getIdentity())
        ) return false;
        
        // Answers accepted are not longer deletable by the answer owner
        return $answer->getSingleFieldValue('questions_answer_accepted', 'score') ? false : true;
    }
    
    public function onEntityCreateContentQuestionsAnswersEntity($bundle, &$values)
    {
        if ($bundle->name !== $this->_answersBundleName) return;
        
        // Initialize field for this entity
        if (!isset($values['content_post_title'])) {
            // Set the current entity as the parent
            $question_id = $values['content_parent'][0]['value'];
            $question = $this->_application->Entity_TypeImpl('content')->entityTypeGetEntityById($question_id);
            $values['content_post_title'] = sprintf(__('Re: %s', 'sabai-discuss'), $question->getTitle());
        }
    }
    
    public function onEntityRenderContentQuestionsHtml($bundle, $entity, $displayMode, $id, &$classes, &$links)
    {
        if ($bundle->name !== $this->_questionsBundleName
            || $displayMode === 'preview'
        ) {
            return;
        }
        
        $answer_count = (int)@$entity->getSingleFieldValue('content_children_count', 'questions_answers');
        if ($displayMode === 'full') {
            // Add status labels
            if ($entity->getSingleFieldValue('questions_resolved')) {
                $classes[] = 'sabai-questions-accepted';
                $entity->data['content_labels']['resolved'] = array(
                    'label' => __('Answered', 'sabai-discuss'),
                    'title' => __('This question has been resolved.', 'sabai-discuss'),
                    'icon' => 'ok',
                    'class' => 'sabai-questions-accepted',
                );
            }
            if ($entity->getSingleFieldValue('questions_closed')) {
                $entity->data['content_labels']['closed'] = array(
                    'label' => __('Closed', 'sabai-discuss'),
                    'title' => __('This question is closed to new answers.', 'sabai-discuss'),
                    'icon' => 'ban-circle',
                    'class' => 'sabai-questions-closed',
                );
            }
            
            $user = $this->_application->getUser();
            $can_manage = $this->_application->HasPermission($this->_config['questions_name'] . '_manage');
            if ($can_manage) {
                $links['edit'] = $this->_application->LinkTo(__('Edit', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/edit'), array('icon' => 'edit'), array('title' => __('Edit this Question', 'sabai-discuss')));
                $links['delete'] = $this->_application->LinkToModal(__('Delete', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/delete', array()), array('width' => 470, 'icon' => 'trash'), array('title' => __('Delete this Question', 'sabai-discuss')));
                $links['close'] = $this->_getCloseQuestionLink($entity);
            } else {
                $is_owner = $this->_application->Content_IsAuthor($entity, $user);
                if ($this->_application->HasPermission($this->_config['questions_name'] . '_edit_any')
                    || ($is_owner && $this->_application->HasPermission($this->_config['questions_name'] . '_edit_own'))
                ) {
                    $links['edit'] = $this->_application->LinkTo(__('Edit', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/edit'), array('icon' => 'edit'), array('title' => __('Edit this Question', 'sabai-discuss')));
                }
                if ($this->isQuestionTrashable($entity, $user)) {
                    $links['delete'] = $this->_application->LinkToModal(__('Delete', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/delete', array()), array('width' => 470, 'icon' => 'trash'), array('title' => __('Delete this Question', 'sabai-discuss')));
                }
                if (($is_owner && $this->_application->HasPermission($this->_config['questions_name'] . '_close_own'))
                    || $this->_application->HasPermission($this->_config['questions_name'] . '_close_any')
                ) {
                    $links['close'] = $this->_getCloseQuestionLink($entity);
                }
            }
        } else {
            // Add title icon
            if ($entity->getSingleFieldValue('questions_resolved')) {
                $classes[] = 'sabai-questions-accepted';
                $entity->data['content_icons']['resolved'] = array(
                    'title' => __('This question has been resolved.', 'sabai-discuss'),
                    'icon' => 'ok-sign',
                    'class' => 'sabai-questions-accepted',
                );
            }
            if ($entity->getSingleFieldValue('questions_closed')) {
                $entity->data['content_icons']['closed'] = array(
                    'title' => __('This question is closed to new answers.', 'sabai-discuss'),
                    'icon' => 'ban-circle',
                    'class' => 'sabai-questions-closed',
                );
            }
        }      
        if (!$entity->getSingleFieldValue('voting_updown', 'count')) {
            $classes[] = 'sabai-questions-novotes';
        }
        if ($answer_count === 0) {
            $classes[] = 'sabai-questions-noanswers';
        }
    }
    
    private function _getCloseQuestionLink($entity)
    {
        if (!$entity->getSingleFieldValue('questions_closed')) {
            $link_label = __('Close', 'sabai-discuss');
            $link_title = __('Close this Question', 'sabai-discuss');
            $active = false;
        } else {
            $link_label = __('Reopen', 'sabai-discuss');
            $link_title = __('Reopen this Question', 'sabai-discuss');            
            $active = true;
        }
        return $this->_application->LinkToModal(
            $link_label,
            $this->_application->Entity_Url($entity, '/close'),
            array(
                'url' => $this->_application->Entity_Url($entity, '/close'),
                'width' => 470,
                'icon' => 'ban-circle',
                'active' => $active,
            ),
            array('title' => $link_title)
        );
    }
    
    public function onEntityRenderContentQuestionsAnswersHtml($bundle, $entity, $displayMode, $id, &$classes, &$links)
    {
        if ($bundle->name !== $this->_answersBundleName
            || $displayMode === 'preview'
        ) {
            return;
        }
        
        $answer_score = (int)$entity->getSingleFieldValue('questions_answer_accepted', 'score');
        if ($answer_score) {
            $classes[] = 'sabai-questions-accepted';
        }
        if ($displayMode === 'full') {
            if ($answer_score) {
                if ($answer_score === 3) {
                    $label = __('Best Answer', 'sabai-discuss');
                } elseif ($answer_score === 2) {
                    $label = __('Great Answer', 'sabai-discuss');
                } elseif ($answer_score === 1) {
                    $label = __('Good Answer', 'sabai-discuss');
                }
                $entity->data['content_labels']['accepted'] = array(
                    'label' => $label,
                    'title' => __('This answer has been accepted.', 'sabai-discuss'),
                    'icon' => 'ok',
                    'class' => 'sabai-questions-accepted',
                );
            }
            $user = $this->_application->getUser();      
            $can_manage = $this->_application->HasPermission($this->_answersBundleName . '_manage');
            if ($can_manage) {
                $links['edit'] = $this->_application->LinkTo(__('Edit', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/edit'), array('icon' => 'edit'), array('title' => __('Edit this Answer', 'sabai-discuss')));
                $links['delete'] = $this->_application->LinkToModal(__('Delete', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/delete', array('delete_target_id' => $id)), array('width' => 470, 'icon' => 'trash'), array('title' => __('Delete this Answer', 'sabai-discuss')));
            } else {
                $is_owner = $this->_application->Content_IsAuthor($entity, $user);
                if ($this->_application->HasPermission($this->_answersBundleName . '_edit_any')
                    || ($is_owner && $this->_application->HasPermission($this->_answersBundleName . '_edit_own'))
                ) {
                    $links['edit'] = $this->_application->LinkTo(__('Edit', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/edit'), array('icon' => 'edit'), array('title' => __('Edit this Answer', 'sabai-discuss')));
                }
                if ($this->isAnswerTrashable($entity, $user)) {
                    $links['delete'] = $this->_application->LinkToModal(__('Delete', 'sabai-discuss'), $this->_application->Entity_Url($entity, '/delete', array('delete_target_id' => $id)), array('width' => 470, 'icon' => 'trash'), array('title' => __('Delete this Answer', 'sabai-discuss')));
                }
            }
            $question = $this->_application->Content_ParentPost($entity, true);
            // Question author can accpet answers
            if ($question
                && ($this->_application->Content_IsAuthor($question, $user)
                    || $this->_application->HasPermission($this->_answersBundleName . '_accept_any'))
            ) {
                $links['accept'] = $this->_application->LinkToModal(
                    $answer_score ? __('Unaccept', 'sabai-discuss') : __('Accept', 'sabai-discuss'),
                    $this->_application->Entity_Url($entity, '/accept', array('update_target_id' => $id)),
                    array('width' => 470, 'icon' => 'ok', 'active' => !empty($answer_score)),
                    array('class' => 'sabai-btn-success', 'title' => $answer_score ? __('Unaccept this Answer', 'sabai-discuss') : __('Accept this Answer', 'sabai-discuss'))
                );
            }
        } else {
            if ($answer_score) {
                // Add title icon
                $entity->data['content_icons']['resolved'] = array(
                    'title' => __('This answer has been accepted.', 'sabai-discuss'),
                    'icon' => 'ok-sign',
                    'class' => 'sabai-questions-accepted',
                );
            }
        }
        if (!$entity->getSingleFieldValue('voting_updown', 'count')) {
            $classes[] = 'sabai-questions-novotes';
        }
    }
    
    public function onVotingQuestionsEntityVotedUpdown(Sabai_Addon_Entity_Entity $entity, $results)
    {
        if ($entity->getBundleName() !== $this->_questionsBundleName) return;
        
        $source_user = $this->_application->getUser()->getIdentity();
        $target_user = $this->_application->Content_Author($entity);
        if ($source_user->id === $target_user->id) return; // no point changes on own content
        
        $source_user_points = $target_user_points = 0;
        $rep_conf = $this->getConfig('reputation');
        
        // Undoing vote?
        if ($results['prev_value'] !== false) {
            $source_user_points += (int)$rep_conf['points']['question_unvoted']; // undoing vote
            if ($results['prev_value'] == 1) {
                $target_user_points -= (int)$rep_conf['points']['question_voted'];
            } elseif ($results['prev_value'] == -1) {
                $target_user_points -= (int)$rep_conf['points']['question_voted_down'];
            }
        }
        // Reflect current vote
        if ($results['value'] == 1) {
            $target_user_points += (int)$rep_conf['points']['question_voted'];           
        } elseif ($results['value'] == -1) {
            $target_user_points += (int)$rep_conf['points']['question_voted_down'];
        }
        
        if ($source_user_points !== 0) {
            $this->_application->Questions_UpdateUserReputation('vote_question', $source_user, $source_user_points, $this->_name);
        }
        if ($target_user_points !== 0) {
            $this->_application->Questions_UpdateUserReputation('question_voted', $target_user, $target_user_points, $this->_name);
        }
    }
    
    public function onVotingQuestionsAnswersEntityVotedUpdown(Sabai_Addon_Entity_Entity $entity, $results)
    {
        if ($entity->getBundleName() !== $this->_answersBundleName) return;
        
        $source_user = $this->_application->getUser()->getIdentity();
        $target_user = $this->_application->Content_Author($entity);
        if ($source_user->id === $target_user->id) return; // no point changes on own content
        
        $source_user_points = $target_user_points = 0;
        $rep_conf = $this->getConfig('reputation');
        
        // Undoing vote?
        if ($results['prev_value'] !== false) {
            if ($results['prev_value'] == 1) {
                $source_user_points += (int)$rep_conf['points']['answer_unvoted']; // undoing an upvote
                $target_user_points -= (int)$rep_conf['points']['answer_voted'];
            } elseif ($results['prev_value'] == -1) {
                $target_user_points -= (int)$rep_conf['points']['answer_voted_down'];
            }
        }
        // Reflect current vote
        if ($results['value'] == 1) {
            $target_user_points += (int)$rep_conf['points']['answer_voted'];            
        } elseif ($results['value'] == -1) {
            $source_user_points += (int)$rep_conf['points']['answer_vote_down'];
            $target_user_points += (int)$rep_conf['points']['answer_voted_down'];
        }
        
        if ($source_user_points !== 0) {
            $this->_application->Questions_UpdateUserReputation('vote_answer', $source_user, $source_user_points, $this->_name);
        }
        if ($target_user_points !== 0) {
            $this->_application->Questions_UpdateUserReputation('answer_voted', $target_user, $target_user_points, $this->_name);
        }
    }    

    public function onEntityCreateContentQuestionsEntitySuccess($bundle, $entity, $values)
    {   
        if ($bundle->name !== $this->_questionsBundleName) {
            return;
        }
        
        if ($entity->isPublished()) {
            $this->_application->Questions_SendQuestionNotification('question_posted', $entity);
        } else {
            $this->_application->Questions_SendQuestionNotification(
                'admin_question_posted',
                $entity,
                null,
                array('{question_url}' => $this->_application->AdminUrl('/' . $this->getQuestionsSlug() . '/' . $entity->getId()))
            );
        }
    }
    
    public function onEntityCreateContentQuestionsAnswersEntitySuccess($bundle, $entity, $values)
    {
        if ($bundle->name !== $this->_answersBundleName
            || !$question = $this->_application->Content_ParentPost($entity)
        ) {
            return;
        }
        
        if ($entity->isPublished()) {
            $this->_application->Questions_SendAnswerNotification('answer_posted', $entity);
            $this->_notifyQuestionAnswered($entity, $question);
        } else {
            $this->_application->Questions_SendAnswerNotification(
                'admin_answer_posted',
                $entity,
                null,
                array('{answer_url}' => $this->_application->AdminUrl('/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers') . '/' . $entity->getId()))
            );
        }
    }
    
    public function onContentQuestionsPostPublished($entity)
    {
        if ($entity->getBundleName() !== $this->_questionsBundleName) return;
        
        $this->_application->Questions_SendQuestionNotification('content_published', $entity, null, array(), 'content_');
        $this->_application->Questions_SendQuestionNotification('question_posted', $entity);
    }
    
    public function onContentQuestionsAnswersPostPublished($entity)
    {
        if ($entity->getBundleName() !== $this->_answersBundleName) return;
        
        $this->_application->Questions_SendAnswerNotification('content_published', $entity, null, array(), 'content_');
        $this->_application->Questions_SendAnswerNotification('answer_posted', $entity);
        // Send question answered notification email
        $this->_notifyQuestionAnswered($entity);
    }
    
    public function onQuestionsAnswerAccepted($question, $answer, $score, $timestamp)
    {
        if ($answer->getBundleName() !== $this->_answersBundleName) return;
        
        $this->_application->Questions_SendAnswerNotification('answer_accepted', $answer, null, array('{acceptance_date}' => $this->_application->DateTime($timestamp)));
    }
            
    public function onCommentSubmitCommentSuccess($comment, $isEdit, $entity)
    {
        if ($isEdit) return;
        
        if ($entity->getBundleName() === $this->_questionsBundleName) {
            $this->_application->Questions_SendQuestionNotification('comment_posted', $entity, null, $this->_application->Comment_TemplateTags($comment), 'content_');
        } elseif ($entity->getBundleName() === $this->_answersBundleName) {
            $this->_application->Questions_SendAnswerNotification('comment_posted', $entity, null, $this->_application->Comment_TemplateTags($comment), 'content_');
        }
    }
        
    private function _notifyQuestionAnswered($answer, $question = null)
    {
        if (!isset($question)
            && (!$question = $this->_application->Content_ParentPost($answer))
        ) {   
            return;
        }
        
        // Send notification if answer author is not the question author
        $question_author = $this->_application->Content_Author($question);
        if ($question_author->email !== $this->_application->Content_Author($answer)->email) {
            $this->_application->Questions_SendAnswerNotification('question_answered', $answer, $question_author);
        }
    }
    
    public function onFormBuildContentAdminListposts(&$form, &$storage)
    {
        if ($form['#bundle']->name !== $this->_questionsBundleName) {
            return;
        }
        
        $form['entities']['#header']['answers'] = array(
            'order' => 12,
            'label' => __('Answers', 'sabai-discuss'),
        );      
        foreach ($form['entities']['#options'] as $entity_id => $data) {
            $entity = $data['#entity'];
            $icons = array();
            if ($entity->getSingleFieldValue('questions_resolved')) {
                $icons[] = '<i class="sabai-content-icon sabai-questions-accepted sabai-icon-ok-sign"></i>';
            }
            if ($entity->getSingleFieldValue('questions_closed')) {
                $icons[] = '<i class="sabai-content-icon sabai-questions-closed sabai-icon-ban-circle"></i>';
            }
            if ($entity->isFeatured()) {
                $icons[] = '<i class="sabai-content-icon sabai-content-featured sabai-icon-certificate"></i>';
            }
            $form['entities']['#options'][$entity_id]['title'] = '<span class="sabai-questions-icons">' . implode(PHP_EOL, $icons) . '</span> ' . $form['entities']['#options'][$entity_id]['title'];
            $form['entities']['#options'][$entity_id] += array(
                'answers' => $this->_application->LinkTo(
                    (int)$entity->getSingleFieldValue('content_children_count', 'questions_answers'),
                    $this->_application->Url('/' . $this->getQuestionsSlug() . '/' . $this->getSlug('answers'), array('content_parent' => $entity_id))
                ),
            );
        }
        $form[Sabai_Addon_Form::FORM_SUBMIT_BUTTON_NAME]['action']['#options'] += array(
            'close' => __('Close', 'sabai-discuss'),
            'reopen' => __('Reopen', 'sabai-discuss'),
        );
        $form['#submit'][0][] = array($this, 'updateQuestions');
        $this->_addAdminListPostsFormHeader($form);
            
        $form['#filters']['questions_answered'] = array(
            'order' => 5,
            'default_option_label' => sprintf(__('Answered / Unanswered', 'sabai-discuss')),
            'options' => array(1 => __('Answered', 'sabai-discuss'), 2 => __('Unanswered', 'sabai-discuss')),
        );
        $form['#filters']['questions_featured'] = array(
            'order' => 6,
            'default_option_label' => sprintf(__('Featured / Unfeatured', 'sabai-discuss')),
            'options' => array(1 => __('Featured', 'sabai-discuss'), 2 => __('Unfeatured', 'sabai-discuss')),
        );
    }
    
    public function onContentAdminPostsUrlParamsFilter(&$urlParams, $context, $bundle)
    {
        if ($bundle->name === $this->_questionsBundleName) {
            if ($questions_answered = $context->getRequest()->asInt('questions_answered')){
                $urlParams['questions_answered'] = $questions_answered;
            }
            if ($questions_featured = $context->getRequest()->asInt('questions_featured')){
                $urlParams['questions_featured'] = $questions_featured;
            }
        } elseif ($bundle->name === $this->_answersBundleName) {
            if ($questions_accepted = $context->getRequest()->asInt('questions_accepted')){
                $urlParams['questions_accepted'] = $questions_accepted;
            }
        }
    }
    
    public function onContentAdminPostsQuery($context, $bundle, $query, $countQuery, $sort, $order)
    {
        if ($bundle->name === $this->_questionsBundleName) {
            if ($questions_answered = $context->getRequest()->asInt('questions_answered')){
                switch ($questions_answered) {
                    case 1:
                        $query->fieldIs('questions_resolved', 1);
                        $countQuery->fieldIs('questions_resolved', 1);
                    break;
                    case 2:
                        $query->fieldIsNull('questions_resolved');
                        $countQuery->fieldIsNull('questions_resolved');
                    break;
                }
            }
            if ($questions_featured = $context->getRequest()->asInt('questions_featured')){
                switch ($questions_featured) {
                    case 1:
                        $query->fieldIs('content_featured', 1);
                        $countQuery->fieldIs('content_featured', 1);
                    break;
                    case 2:
                        $query->fieldIsNull('content_featured');
                        $countQuery->fieldIsNull('content_featured');
                    break;
                }
            }
        } elseif ($bundle->name === $this->_answersBundleName) {
            if ($questions_accepted = $context->getRequest()->asInt('questions_accepted')){
                switch ($questions_accepted) {
                    case 1:
                        $query->fieldIsOrGreaterThan('questions_answer_accepted', 1, 'score');
                        $countQuery->fieldIsOrGreaterThan('questions_answer_accepted', 1, 'score');
                    break;
                    case 2:
                        $query->fieldIsNull('questions_answer_accepted');
                        $countQuery->fieldIsNull('questions_answer_accepted');
                    break;
                }
            }
        }
    }
    
    public function onFormBuildContentAdminListchildposts(&$form, &$storage)
    {
        if ($form['#bundle']->name !== $this->_answersBundleName) {
            return;
        }
        $form['entities']['#header']['title'] = array(
            'label' => __('Answer', 'sabai-discuss'),
            'order' => 1,
        );
        foreach ($form['entities']['#options'] as $entity_id => $data) {
            $entity = $data['#entity'];
            $icon = $entity->getSingleFieldValue('questions_answer_accepted', 'score') ? '<i class="sabai-content-icon sabai-questions-accepted sabai-icon-ok-sign"></i> ' : '';
            $form['entities']['#options'][$entity_id]['title'] = $icon . Sabai::h($entity->getSummary(200)) . '<div class="sabai-row-action">' . $this->_application->Menu($data['#links']) . '</div>';
        }
        $this->_addAdminListPostsFormHeader($form);
        
        $form['#filters']['questions_accepted'] = array(
            'order' => 5,
            'default_option_label' => sprintf(__('Show accepted/unaccepted', 'sabai-discuss')),
            'options' => array(1 => __('Show accepted', 'sabai-discuss'), 2 => __('Show unaccepted', 'sabai-discuss')),
        );
    }
    
    private function _addAdminListPostsFormHeader(&$form)
    {
        if ($form['#status'] === 'trashed') {
            $form['#header'] = array(
                '<p>' . sprintf(__('Posts <span class="sabai-tr-error">highlighted</span> are marked as SPAM. Authors of these posts will lose %d reputation points when the posts are removed from trash.', 'sabai-discuss'), abs($this->_config['reputation']['points']['spam'])) . '</p>'
            );
        }
    }
    
    public function updateQuestions(Sabai_Addon_Form_Form $form)
    {
        if (!empty($form->values['entities'])) {
            switch ($form->values['action']) {
                case 'close':
                    $this->_closeQuestions($form->values['entities']);
                    break;
                case 'reopen':
                    $this->_closeQuestions($form->values['entities'], false);
                    break;
            }
        }
    }
    
    protected function _closeQuestions($entities, $close = true)
    {
        foreach ($this->_application->Entity_TypeImpl('content')->entityTypeGetEntitiesByIds($entities) as $entity) {
            $this->_application->getAddon('Entity')->updateEntity($entity, array('questions_closed' => $close));
        }
    }
    
    public function onSystemLoadPermissions($identity, &$permissions)
    {
        if (!$this->_config['perm_rep_enable'] || $identity->isAnonymous()) return;
        
        $user_roles = $this->_application->getPlatform()->getUserRolesByUser($identity);
        $user_reputation = $this->_application->Questions_UserReputation($identity, $this->_name);
        foreach ($user_roles as $user_role) {
            if (!isset($this->_config['perm_rep'][$user_role])) continue;
            // Add permissions that have minimum reputation lower or equal to the current user reputation
            foreach ($this->_config['perm_rep'][$user_role] as $permission => $reputation) {
                if ($user_reputation >= $reputation) {
                    $permissions[$permission] = 1;
                } else {
                    unset($permissions[$permission]);
                }
            }
        }
    }
    
    public function onEntityBulkDeleteContentQuestionsEntitySuccess($bundle, $entities, $extra)
    {
        if ($bundle->name !== $this->_questionsBundleName) return;
        
        $this->_onEntityBulkDeleteContentEntitySuccess($bundle, $entities, $extra);
    }
    
    public function onEntityBulkDeleteContentQuestionsAnswersEntitySuccess($bundle, $entities, $extra)
    {
        if ($bundle->name !== $this->_answersBundleName) return;
        
        $this->_onEntityBulkDeleteContentEntitySuccess($bundle, $entities, $extra);
    }
        
    private function _onEntityBulkDeleteContentEntitySuccess($bundle, $entities, $extra)
    {
        // Subtract reputation points from authors of spam posts
        foreach ($entities as $post) {
            if ($post->getSingleFieldValue('content_trashed', 'type') === Sabai_Addon_Content::TRASH_TYPE_SPAM) {
                $this->_application->Questions_UpdateUserReputation('content_deleted', $this->_application->Content_Author($post), $this->_config['reputation']['points']['spam'], $this->_name);
            }
        }
    }
    
    public function onQuestionsInstallSuccess($addon)
    {
        if ($addon->getName() !== $this->_name) return;
        
        $this->_application->Questions_CreateSampleData($addon->getName());
    }
    
    public function onVotingQuestionsEntityVotedFlag(Sabai_Addon_Entity_IEntity $entity, $results, $vote)
    {
        if ($entity->getBundleName() !== $this->_questionsBundleName) return;
        
        $this->_application->Questions_SendQuestionNotification('question_flagged', $entity, null, array('{flag_score_total}' => (int)$results['sum']) + $this->_application->Voting_TemplateTags($vote));
        $this->_trashPostIfSpam($entity, $results);
    }
    
    public function onVotingQuestionsAnswersEntityVotedFlag(Sabai_Addon_Entity_IEntity $entity, $results, $vote)
    {
        if ($entity->getBundleName() !== $this->_answersBundleName) return;
        
        $this->_application->Questions_SendAnswerNotification('answer_flagged', $entity, null, array('{flag_score_total}' => (int)$results['sum']) + $this->_application->Voting_TemplateTags($vote));
        $this->_trashPostIfSpam($entity, $results);
    }
    
    private function _trashPostIfSpam(Sabai_Addon_Entity_IEntity $entity, $results)
    {
        if ($entity->isTrashed()) return; // trashed posts can not be flagged, but just in case

        // Has the spam score reached the threshold?
        if ($results['sum'] > $this->_config['spam']['threshold'] + 0.3 * (int)$entity->getSingleFieldValue('voting_updown', 'sum')) {
            // Move to trash and clear flags
            $this->_application->Content_TrashPosts($entity, Sabai_Addon_Content::TRASH_TYPE_SPAM, '', 0);
        }
    }
    
    public function onSabaiRunCron($lastRunTimestamp, $logs)
    {
        if (!$this->_config['spam']['auto_delete']) {
            // Auto-delete spam not enabled
            return;
        }
        
        // Fetch posts marked as spam and were trashed more than X days ago
        $days = $this->_config['spam']['delete_after'];
        $spam_posts = $this->_application->Entity_Query('content')
            ->propertyIsIn('post_entity_bundle_name', array($this->_questionsBundleName, $this->_answersBundleName))
            ->propertyIs('post_status', Sabai_Addon_Content::POST_STATUS_TRASHED) // trashed posts
            ->fieldIs('content_trashed', Sabai_Addon_Content::TRASH_TYPE_SPAM, 'type') // marked as spam
            ->fieldIsOrSmallerThan('content_trashed', time() - $days * 86400, 'trashed_at') // more than X days after trashed
            ->fetch(0, 0, false);    
        if (empty($spam_posts)) {
            return;
        }
        // Delete
        $this->_application->Content_DeletePosts($spam_posts);
        $logs[] = sprintf(
            __('Deleted %d spam posts (questions and/or answers) from trash', 'sabai-discuss'),
            count($spam_posts)
        );
    }
    
    private function _isAllowedAccess()
    {
        if (!isset($this->_allowedAccess)) {
            $this->_allowedAccess = $this->_checkAllowedAccess();
        }
        return $this->_allowedAccess;
    }
    
    private function _checkAllowedAccess()
    {
        if ($this->_config['access']['type'] === 'closed') {
            return false;
        }
        if ($this->_config['access']['type'] === 'public'
            || $this->_application->getUser()->isAdministrator()
        ) {
            return true;
        }
        // Deny non-registered users
        if ($this->_application->getUser()->isAnonymous()) {
            return false;
        }
        // Allow selected roles only?
        if (!empty($this->_config['access']['roles'])) {
            foreach ($this->_application->getPlatform()->getUserRolesByUser($this->_application->getUser()->getIdentity()) as $role) {
                if (in_array($role, $this->_config['access']['roles'])) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public function getInstallForm(array $parents = array())
    {
        return $this->getPageTitleSettingsForm() + $this->getLabelSettingsForm() + $this->getSlugSettingsForm($parents);
    }
    
    public function getUpgradeForm($currentVersion, array $parents = array())
    {
        return array();
    }
    
    public function getPageTitleSettingsForm($collapsed = false)
    {
        return array(
            'page_title' => array(
                '#title' => __('Page Title Settings', 'sabai-discuss'),
                '#collapsed' => $collapsed,
                'main' => array(
                    '#title' => __('Main index page', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#default_value' => $this->getPageTitle(),
                    '#required' => true,
                    '#size' => 40,
                ),
                'category' => array(
                    '#title' => __('Single category page', 'sabai-discuss'),
                    '#description' => '%s represents the title of category',
                    '#type' => 'textfield',
                    '#default_value' => isset($this->_config['page_title']['category']) ? $this->_config['page_title']['category'] : '%s',
                    '#required' => true,
                    '#size' => 40,
                ),
                'tag' => array(
                    '#title' => __('Single tag page', 'sabai-discuss'),
                    '#description' => '%s represents the title of tag',
                    '#type' => 'textfield',
                    '#default_value' => isset($this->_config['page_title']['tag']) ? $this->_config['page_title']['tag'] : '%s',
                    '#required' => true,
                    '#size' => 40,
                ),
            ),
        );
    }
    
    public function getLabelSettingsForm($collapsed = false)
    {
        return array(
            'label' => array(
                '#title' => __('Label Settings', 'sabai-discuss'),
                '#collapsed' => $collapsed,
                'categories' => array(
                    '#title' => __('Categories', 'sabai-discuss'),
                    '#description' => sprintf(__('Enter the label to be used for "%s"', 'sabai-discuss'), __('Categories', 'sabai-discuss')),
                    '#type' => 'textfield',
                    '#default_value' => @$this->_config['label']['categories'],
                    '#required' => true,
                    '#size' => 40,
                ),
                'category' => array(
                    '#title' => __('Category', 'sabai-discuss'),
                    '#description' => sprintf(__('Enter the label to be used for "%s"', 'sabai-discuss'), __('Category', 'sabai-discuss')),
                    '#type' => 'textfield',
                    '#default_value' => @$this->_config['label']['category'],
                    '#required' => true,
                    '#size' => 40,
                ),
                'tags' => array(
                    '#title' => __('Tags', 'sabai-discuss'),
                    '#description' => sprintf(__('Enter the label to be used for "%s"', 'sabai-discuss'), __('Tags', 'sabai-discuss')),
                    '#type' => 'textfield',
                    '#default_value' => @$this->_config['label']['tags'],
                    '#required' => true,
                    '#size' => 40,
                ),
                'tag' => array(
                    '#title' => __('Tag', 'sabai-discuss'),
                    '#description' => sprintf(__('Enter the label to be used for "%s"', 'sabai-discuss'), __('Tag', 'sabai-discuss')),
                    '#type' => 'textfield',
                    '#default_value' => @$this->_config['label']['tag'],
                    '#required' => true,
                    '#size' => 40,
                ),
            ),
        );
    }
    
    public function getSlugSettingsForm(array $parents = array(), $collapsed = false)
    {
        array_push($parents, 'slug');
        return array(
            'slug' => array(
                '#title' => __('Slug Settings', 'sabai-discuss'),
                '#collapsed' => $collapsed,
                'questions' => array(
                    '#title' => __('Questions slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('questions'),
                    '#required' => true,
                    '#description' => __('This is the root slug that will prefix all content. Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#element_validate' => array(array($this, 'validateBaseSlug')),
                    '#regex' => '/^[a-z0-9-_]+$/',
                    '#max_length' => 25,
                ),
                'question' => array(
                    '#title' => __('Single question slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('question'),
                    '#required' => true,
                    '#description' => __('Slug used for the single question page. Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#element_validate' => array(array(array($this, 'validateSlug'), array('question', $parents))),
                    '#regex' => '/^[a-z0-9-_]+$/',
                ),
                'answers' => array(
                    '#title' => __('Answers slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('answers'),
                    '#required' => true,
                    '#description' => __('Slug used for the answer archive page. Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#regex' => '/^[a-z0-9-_]+$/',
                    '#element_validate' => array(array(array($this, 'validateSlug'), array('answers', $parents))),
                    '#max_length' => 13,
                ),
                'ask' => array(
                    '#title' => __('Ask question slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('ask'),
                    '#required' => true,
                    '#description' => __('Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#regex' => '/^[a-z0-9-_]+$/',
                    '#element_validate' => array(array(array($this, 'validateSlug'), array('ask', $parents))),
                    '#max_length' => 13,
                ),
                'tags' => array(
                    '#title' => __('Question tag slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('tags'),
                    '#required' => true,
                    '#description' => __('Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#element_validate' => array(array(array($this, 'validateSlug'), array('tags', $parents))),
                    '#regex' => '/^[a-z0-9-_]+$/',
                    '#max_length' => 13,
                ),
                'categories' => array(
                    '#title' => __('Question category slug', 'sabai-discuss'),
                    '#type' => 'textfield',
                    '#size' => 40,
                    '#default_value' => $this->getSlug('categories'),
                    '#required' => true,
                    '#description' => __('Must consist of lower-case alphanumeric characters, dashes, and underscores only.', 'sabai-discuss'),
                    '#element_validate' => array(array(array($this, 'validateSlug'), array('categories', $parents))),
                    '#regex' => '/^[a-z0-9-_]+$/',
                    '#max_length' => 13,
                ),
            ),
        );
    }
    
    public function validateBaseSlug(Sabai_Addon_Form_Form $form, &$value, $element)
    {
        if (in_array($value, array('sabai'))) {
            $form->setError(__('The slug is reserved by the system.', 'sabai-discuss'), $element);
            return;
        }
        if ($this->_application->getModel('Bundle', 'Entity')->name_isNot($this->_config['questions_name'])->path_is('/' . $value)->count()) {
            $form->setError(__('The slug is already in use by another add-on.', 'sabai-discuss'), $element);
            return;
        }
    }
    
    public function validateSlug(Sabai_Addon_Form_Form $form, &$value, $element, $self, $parents)
    {
        if (in_array($value, array('add', 'feed', 'users'))) {
            $form->setError(__('The slug is reserved by the system.', 'sabai-discuss'), $element);
            return;
        }
        $slug_values = $form->getValue($parents);
        foreach (array('questions', 'question', 'answers', 'ask', 'tags', 'categories') as $slug) {
            if ($slug !== $self && array_key_exists($slug, $slug_values) && $slug_values[$slug] === $value) {
                $form->setError(__('The slug must be a unique value.', 'sabai-discuss'), $element);
                return;
            }
        }
    }
    
    public function onQuestionsUpgradeSuccess(Sabai_Addon $addon, $log, $previousVersion)
    {
        if ($addon->getName() !== $this->_name
            || $this->hasParent() // Cloned add-ons should not proceed
        ) {
            return;
        }
        require_once $this->_path . '/includes/upgrade.php';
        sabai_addon_questions_upgrade($this->_application, $previousVersion);
    }
    
    public function onContentTitleFilter(&$title, $bundleName)
    {
        if ($bundleName === $this->_categoriesBundleName) {
            if (isset($this->_config['page_title']['category'])) {
                $title = sprintf($this->_config['page_title']['category'], $title);
            }
        } elseif ($bundleName === $this->_tagsBundleName) {
            if (isset($this->_config['page_title']['tag'])) {
                $title = sprintf($this->_config['page_title']['tag'], $title);
            }
        }
    }
    
    public function onEntityBundleLabelFilter(&$label, $bundleName, $singular)
    {
        if ($bundleName === $this->_categoriesBundleName) {
            $key = $singular ? 'category' : 'categories';
            if (!empty($this->_config['label'][$key])) {
                $label = $this->_config['label'][$key];
            }
        } elseif ($bundleName === $this->_tagsBundleName) {
            $key = $singular ? 'tag' : 'tags';
            if (!empty($this->_config['label'][$key])) {
                $label = $this->_config['label'][$key];
            }
        }
    }
    
    public function onSystemUserProfileActivityFilter(&$activity, $identity, $counts)
    {
        if (!$this->_isAllowedAccess()) {
            return;
        }
        $activity[] = array(            
            'stats' => array(
                $this->_questionsBundleName => array(
                    'url' => '/' . $this->getQuestionsSlug() . '/users/' . $identity->username,
                    'format' => _n('%s question', '%s questions', $count = isset($counts[$this->_questionsBundleName]) ? $counts[$this->_questionsBundleName] : 0, 'sabai-discuss'),
                    'is_bundle' => true,
                    'count' => $count,
                ),
                $this->_answersBundleName => array(
                    'url' => '/' . $this->getQuestionsSlug() . '/users/' . $identity->username . '/' . $this->getSlug('answers'),
                    'format' => _n('%s answer', '%s answers', $count = isset($counts[$this->_answersBundleName]) ? $counts[$this->_answersBundleName] : 0, 'sabai-discuss'),
                    'is_bundle' => true,
                    'count' => $count,
                ),
                'reputation' => array(
                    'format' => __('%s reputation', 'sabai-discuss'),
                    'count' => $this->_application->Questions_UserReputation($identity, $this->_name),
                ),
            ),
            'title' => $this->getPageTitle(),
        );
    }
    
    public function onSabaiWebResponseRenderHtmlLayout(Sabai_Context $context, Sabai_WebResponse $response, &$content)
    {
        if ($this->hasParent()) return;
        
        // The main stylesheet should already have been included by the platform if not requesting the full page content
        if ($context->getContainer() === '#sabai-content') {
            if (!$last_update = $this->_application->getAddonsLoadedTimestamp()) {
                $last_update = time();
            }
            $css_file = $context->isAdmin() ? 'admin.css' : 'main.css';
            $this->_application->LoadCss($response, $this->_application->getPlatform()->getAssetsUrl('sabai-discuss') . '/css/' . $css_file, 'sabai-discuss', $last_update);
            if (!$context->isAdmin() && $this->_application->getPlatform()->isLanguageRTL()) {
                $this->_application->LoadCss($response, $this->_application->getPlatform()->getAssetsUrl('sabai-discuss') . '/css/main-rtl.css', 'sabai-discuss-rtl', $last_update);
            }
        }
    }
}
