<?php
class Sabai_Addon_Questions_Controller_Admin_Settings extends Sabai_Addon_Form_Controller
{    
    protected function _doGetFormSettings(Sabai_Context $context, array &$formStorage)
    {
        $config = $this->getAddon()->getConfig();
        $this->_cancelUrl = null;
        $this->_submitButtons[] = array('#value' => __('Save Changes', 'sabai-discuss'), '#btn_type' => 'primary');
        $this->_successFlash = __('Settings saved.', 'sabai-discuss');
        $form = array('#tree' => true);
        $form['front'] = array(
            '#title' => __('Display Settings', 'sabai-discuss'),
            'perpage' => array(
                '#type' => 'textfield',
                '#title' => __('Number of posts per page', 'sabai-discuss'),
                '#default_value' => $config['front']['perpage'],
                '#size' => 6,
                '#integer' => true,
                '#required' => true,
                '#max_value' => 100,
                '#min_value' => 1,
                '#field_suffix' => sprintf(__('(max. limit %d)', 'sabai-discuss'), 100),
            ),
            'sort' => array(
                '#type' => 'radios',
                '#default_value' => $config['front']['sort'],
                '#title' => __('Default sorting order', 'sabai-discuss'),
                '#options' => array(
                    'newest' => __('Newest posts first', 'sabai-discuss'),
                    'active' => __('Most active posts first', 'sabai-discuss'),
                    'votes' => __('Most voted posts first', 'sabai-discuss'),
                ),
                '#class' => 'sabai-form-inline',
                '#required' => true,
            ),
            'feature' => array(
                '#type' => 'checkbox',
                '#title' => __('Stick featured posts to the top', 'sabai-discuss'),
                '#default_value' => !empty($config['front']['feature']),
            ),
            'categories_tab' => array(
                '#type' => 'checkbox',
                '#title' => __('Show the "Categories" tab', 'sabai-discuss'),
                '#default_value' => !empty($config['front']['categories_tab']),
            ),
            'tags_tab' => array(
                '#type' => 'checkbox',
                '#title' => __('Show the "Tags" tab', 'sabai-discuss'),
                '#default_value' => !empty($config['front']['tags_tab']),
            ),
            'buttons' => array(
                '#title' => __('Buttons', 'sabai-discuss'),
                '#class' => 'sabai-form-group',
                '#collapsible' => false,
                'search' => array(
                    '#class' => 'sabai-form-inline',
                    '#type' => 'radios',
                    '#options' => $this->ButtonOptions('<i class="sabai-icon-search"></i>'),
                    '#title_no_escape' => true,
                    '#default_value' => isset($config['front']['buttons']['search']) ? $config['front']['buttons']['search'] : 'sabai-btn-primary',
                ),
                'ask' => array(
                    '#class' => 'sabai-form-inline',
                    '#type' => 'radios',
                    '#options' => $this->ButtonOptions('<i class="sabai-icon-pencil"></i> ' . __('Ask Question', 'sabai-discuss')),
                    '#title_no_escape' => true,
                    '#default_value' => isset($config['front']['buttons']['ask']) ? $config['front']['buttons']['ask'] : 'sabai-btn-success',
                ),
                'answer' => array(
                    '#class' => 'sabai-form-inline',
                    '#type' => 'radios',
                    '#options' => $this->ButtonOptions('<i class="sabai-icon-pencil"></i> ' . __('Post Answer', 'sabai-discuss')),
                    '#title_no_escape' => true,
                    '#default_value' => isset($config['front']['buttons']['answer']) ? $config['front']['buttons']['answer'] : 'sabai-btn-success',
                ),
            ),
        );
        $form['reputation'] = array(
            '#title' => __('Reputation Settings', 'sabai-discuss'),
            'info' => array(
                '#weight' => 1,
                '#type' => 'markup',
                '#markup' => '<p>' . __('Reputation is a rough measurement of how much the community trusts you; it is earned by convincing your peers that you know what you\'re talking about. Here you can configure reputation users gain or lose on certain events or actions.', 'sabai-discuss') . '</p>',
            ),
            'points' => array(
                '#weight' => 2,
                '#class' => 'sabai-form-group',
                'question_voted' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['question_voted']) ? $config['reputation']['question_voted'] : @$config['reputation']['points']['question_voted'],
                    '#field_prefix' => __('One of your questions is voted up:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'question_voted_down' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['question_voted_down']) ? $config['reputation']['question_voted_down'] : @$config['reputation']['points']['question_voted_down'],
                    '#field_prefix' => __('One of your questions is voted down:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_voted' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['answer_voted']) ? $config['reputation']['answer_voted'] : @$config['reputation']['points']['answer_voted'],
                    '#field_prefix' => __('One of your answers is voted up:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_voted_down' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['answer_voted_down']) ? $config['reputation']['answer_voted_down'] : @$config['reputation']['points']['answer_voted_down'],
                    '#field_prefix' => __('One of your answers is voted down:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_accepted' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['answer_accepted']) ? $config['reputation']['answer_accepted'] : @$config['reputation']['points']['answer_accepted'],
                    '#field_prefix' => __('One of your answers become accepted:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_accepted_user' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['answer_accepted_user']) ? $config['reputation']['answer_accepted_user'] : @$config['reputation']['points']['answer_accepted_user'],
                    '#field_prefix' => __('You accept an answer posted to your question:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_vote_down' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['answer_vote_down']) ? $config['reputation']['answer_vote_down'] : @$config['reputation']['points']['answer_vote_down'],
                    '#field_prefix' => __('You vote an answer down:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'question_unvoted' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['points']['question_unvoted']) ? $config['reputation']['points']['question_unvoted'] : -1,
                    '#field_prefix' => __('You unvote a question:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'answer_unvoted' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['points']['answer_unvoted']) ? $config['reputation']['points']['answer_unvoted'] : -1,
                    '#field_prefix' => __('You cancel an upvote on an answer:', 'sabai-discuss'),
                    '#size' => 6,
                    '#integer' => true,
                    '#required' => true,
                ),
                'spam' => array(
                    '#type' => 'textfield',
                    '#default_value' => isset($config['reputation']['spam']) ? $config['reputation']['spam'] : @$config['reputation']['points']['spam'],
                    '#field_prefix' => __('One of your questions or answers is deleted as being spam:', 'sabai-discuss'),
                    '#size' => 5,
                    '#integer' => true,
                    '#required' => true,
                ),
            ),
        );
        $form['search'] = array(
            '#title' => __('Search Settings', 'sabai-discuss'),
            'min_keyword_len' => array(
                '#type' => 'textfield',
                '#title' => __('Min. length of keywords in characters', 'sabai-discuss'),
                '#size' => 3,
                '#default_value' => isset($config['search']['min_keyword_len']) ? $config['search']['min_keyword_len'] : 3,
                '#integer' => true,
                '#required' => true,
                '#min_value' => 1,
            ),
        );
        $form['spam'] = array(
            '#title' => __('Spam Settings', 'sabai-discuss'),
            'threshold' => array(
                '#type' => 'textfield',
                '#title' => __('Spam score threshold', 'sabai-discuss'),
                '#field_suffix' => '+ 0.3 x ' . __('number of votes', 'sabai-discuss'),
                '#description' => __('When a post is flagged, the post is assigned a "spam score". Posts with spam scores exceeding the threshold value are marked as spam and moved to trash automatically by the system. Also, posts with higher number of votes will have higher threshold. For example, if the value set here is 11, and a post has 10 votes, then the spam score threshold for the post will be 14 (11 + 0.3 x 10).', 'sabai-discuss'),
                '#default_value' => $config['spam']['threshold'],
                '#size' => 4,
                '#integer' => true,
                '#required' => true,
            ),
            'auto_delete' => array(
                '#type' => 'checkbox',
                '#title' => __('Auto-delete spam', 'sabai-discuss'),
                '#default_value' => $config['spam']['auto_delete'],
                '#description' => __('When checked, posts that have been marked as spam will be deleted by the system after the period of time specified in the "Delete Spam After" option.', 'sabai-discuss'),
            ),
            'delete_after' => array(
                '#type' => 'textfield',
                '#default_value' => $config['spam']['delete_after'],
                '#field_prefix' => __('Delete spam after:', 'sabai-discuss'),
                '#description' => __('Enter the number of days the system will wait before auto-deleting posts marked as spam.', 'sabai-discuss'),
                '#field_suffix' => __('days', 'sabai-discuss'),
                '#size' => 4,
                '#integer' => true,
                '#states' => array(
                    'visible' => array(
                        'input[name="spam[auto_delete][]"]' => array('type' => 'checked', 'value' => true),
                    ),
                ),
            ),
        );
        $form += $this->getAddon()->getPageTitleSettingsForm(true);
        $form += $this->getAddon()->getLabelSettingsForm(true);
        $form += $this->getAddon()->getSlugSettingsForm(array(), true);
        
        return $form;
    }

    public function submitForm(Sabai_Addon_Form_Form $form, Sabai_Context $context)
    {
        $new_config = array(
            'front' => $form->values['front'],
            'reputation' => $form->values['reputation'],
            'spam' => $form->values['spam'],
            'page_title' => $form->values['page_title'],
            'label' => $form->values['label'],
            'slug' => $form->values['slug'],
            'search' => $form->values['search'],
        );
        
        // Run upgrade process to refresh all slug related data
        $log = new ArrayObject();
        $addon = $this->UpgradeAddon($this->getAddon()->getName(), $new_config, $log);
        $this->reloadAddons()
            ->doEvent('SabaiAddonUpgraded', array($addon, $addon->version, $log));
        $this->getPlatform()->clearCache();
        $context->setSuccess('/' . $this->getAddon()->getQuestionsSlug() . '/settings');
    }
}
