<?php
class Sabai_Addon_Questions_Controller_Questions extends Sabai_Addon_Content_Controller_ListPosts
{
    protected $_template = 'questions_questions', $_sortContainer = '#sabai-questions-questions', $_scrollTo = '#sabai-questions-questions', $_settings;
    
    protected function _doExecute(Sabai_Context $context)
    {
        $this->_settings = $this->_getDefaultSettings($context);
        $this->_perPage = $this->_settings['perpage'];
        $this->_defaultSort = $this->_settings['sort'];
        // Init filters
        $this->_settings['filters'] = array(
            '' => __('All Questions', 'sabai-discuss'),
            'resolved' => __('Resolved Questions', 'sabai-discuss'),
            'unresolved' => __('Unresolved Questions', 'sabai-discuss'),
            'closed' => __('Closed Questions', 'sabai-discuss'),
            'open' => __('Open Questions', 'sabai-discuss'),
        );
        
        if ($keywords = $context->getRequest()->asStr('keywords', $this->_settings['keywords'])) {
            $this->_settings['keywords'] = $this->Keywords($keywords, $this->getAddon()->getConfig('search', 'min_keyword_len'));
        }
        $this->_settings['category'] = $context->getRequest()->asInt('category', $this->_settings['parent_category']);
        $this->_settings['filter'] = $context->getRequest()->asStr('filter', $this->_settings['filter'], array_keys($this->_settings['filters']));
        parent::_doExecute($context);
        $filters = array();
        foreach ($this->_settings['filters'] as $filter => $label) {
            $filters[$filter] = $this->LinkToRemote(
                $label,
                '#sabai-questions-questions',
                $this->Url($context->getRoute(), array('filter' => $filter) + $context->url_params),
                array('active' => $this->_settings['filter'] === $filter, 'scroll' => '#sabai-questions-questions')
            );
        }
        $context->setAttributes(array(
            'settings' => $this->_settings,
            'filters' => $filters,
        ));
        // Load partial content if request is ajax
        if ($context->getRequest()->isAjax() === '#sabai-questions-questions') {
            if (!$context->entities) {
                $context->addTemplate('questions_questions_none');
            } else {
                $context->addTemplate('questions_questions_list');
            }
        }
    }

    protected function _getUrlParams(Sabai_Context $context, Sabai_Addon_Entity_Model_Bundle $bundle = null)
    {
        $params = array();
        if (!empty($this->_settings['keywords'][0])) {
            $params['keywords'] = implode(' ', $this->_settings['keywords'][0]);
        }
        if (!empty($this->_settings['category'])) {
            $params['category'] = $this->_settings['category'];
        }
        $params['filter'] = $this->_settings['filter'];
        
        return $params;
    }

    protected function _getSorts(Sabai_Context $context)
    {  
        $sorts = array(
            'newest' => __('Newest First', 'sabai-discuss'),
            'oldest' => __('Oldest First', 'sabai-discuss'),
            'active' => __('Recently Active', 'sabai-discuss'),
            'votes' => __('Most Votes', 'sabai-discuss'),
        );
        if (empty($this->_settings['keywords'])) {
            $sorts += array('views' => __('Most Viewed', 'sabai-discuss'));
        }
        return $sorts;
    }
    
    protected function _paginate(Sabai_Context $context, $sort, Sabai_Addon_Entity_Model_Bundle $bundle = null)
    {
        if (empty($this->_settings['keywords'])) {
            return parent::_paginate($context, $sort, $bundle);
        }
        return $this->Questions_PaginateSearch($bundle, $this->getAddon()->getAnswersBundleName(), $this->_settings['keywords'], $this->_settings['category'], $this->_settings['tag'], $sort, $this->_settings['filter'], $this->_perPage)
            ->setCurrentPage($context->getRequest()->asInt(Sabai::$p, $this->_defaultPage));;
    }
    
    protected function _createQuery(Sabai_Context $context, $sort, Sabai_Addon_Entity_Model_Bundle $bundle = null)
    {
        return $this->Questions_Query(
            parent::_createQuery($context, $sort, $bundle),
            $this->_settings['category'],
            $this->_settings['tag'],
            $sort,
            $this->_settings['filter'],
            $this->_settings['feature'],
            $this->_settings['featured_only']
        );
    }
    
    protected function _getDefaultSettings(Sabai_Context $context)
    {
        $config = $this->getAddon()->getConfig();
        return array(
            'perpage' => $config['front']['perpage'],
            'sort' => $config['front']['sort'],
            'keywords' => array(),
            'parent_category' => $this->_getDefaultCategoryId($context),
            'category_bundle' => $this->getAddon()->getCategoriesBundleName(),
            'feature' => $config['front']['feature'],
            'featured_only' => false,
            'filter' => '',
            'tag' => $this->_getDefaultTagId($context),
            'buttons' => $config['front']['buttons'],
        );
    }
    
    protected function _getDefaultCategoryId(Sabai_Context $context)
    {
        return 0;
    }
    
    protected function _getDefaultTagId(Sabai_Context $context)
    {
        return null;
    }
    
    protected function _getLinks(Sabai_Context $context, $sort, Sabai_Addon_Entity_Model_Bundle $bundle = null)
    {
        if (!isset($bundle)) {
            return array();
        }
        if (!empty($this->_settings['parent_category'])) {
            $term_id = $this->_settings['parent_category'];
        } elseif (!empty($this->_settings['tag'])) {
            $term_id = $this->_settings['tag'];
        }
        return array(
            $this->LinkTo(
                __('Ask Question', 'sabai-discuss'),
                $this->Url($bundle->getPath() . '/' . $this->getAddon()->getSlug('ask'), isset($term_id) ? array('term_id' => $term_id) : array()),
                array('icon' => 'pencil'),
                array('class' => 'sabai-btn sabai-btn-small ' . $this->getAddon()->getConfig('front', 'buttons', 'ask'))
            ),
        );
    }
}
