<?php
class Sabai_Addon_Questions_Helper_PaginateSearch extends Sabai_Helper
{
    protected $_countSql, $fetchSql;
    
    public function help(Sabai $application, $questionBundle, $answerBundleName, array $keywords, $category, $tag, $sort, $filter, $limit)
    {
        $db = $application->getDB();        
        $fetch_sql1 = 'SELECT p.post_id AS id, %2$s AS sorter FROM %1$scontent_post p
            LEFT JOIN %1$sentity_field_content_body b ON b.entity_id = p.post_id';
        $fetch_sql2 = 'SELECT pa.value AS id, %3$s AS sorter FROM %1$scontent_post p 
            LEFT JOIN %1$sentity_field_content_parent pa ON pa.entity_id = p.post_id
            LEFT JOIN %1$sentity_field_content_body b ON b.entity_id = p.post_id';
        $count_sql1 = 'SELECT p.post_id AS id FROM %1$scontent_post p
            LEFT JOIN %1$sentity_field_content_body b ON b.entity_id = p.post_id';
        $count_sql2 = 'SELECT pa.value AS id FROM %1$scontent_post p 
            LEFT JOIN %1$sentity_field_content_parent pa ON pa.entity_id = p.post_id
            LEFT JOIN %1$sentity_field_content_body b ON b.entity_id = p.post_id';
        
        $where1 = $where2 = array('p.post_status = ' . $db->escapeString(Sabai_Addon_Content::POST_STATUS_PUBLISHED));
        $where1[] = 'p.post_entity_bundle_name = ' . $db->escapeString($questionBundle->name);
        $where2[] = 'p.post_entity_bundle_name = ' . $db->escapeString($answerBundleName);
        
        // Add keywords
        if (!empty($keywords[0])) {
            foreach ($keywords[0] as $keyword) {
                $keyword_escaped = $db->escapeString('%' . $keyword . '%');
                $where1[] = sprintf('(p.post_title LIKE %1$s OR b.value LIKE %1$s)', $keyword_escaped);
                $where2[] = 'b.value LIKE ' . $keyword_escaped;
            }
        }
        // Add categories?
        if (!empty($category)) {
            $category_ids = array((int)$category);
            foreach ($application->Taxonomy_Descendants($category) as $_category) {
                $category_ids[] = $_category->id;
            }
            $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_categories c ON c.entity_id = p.post_id';
            $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_categories c ON c.entity_id = pa.value';
            $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_categories c ON c.entity_id = p.post_id';
            $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_categories c ON c.entity_id = pa.value';
            if (count($category_ids) > 1) {
                $where1[] = $where2[] = sprintf('c.value IN (%s)', implode(',', $category_ids));
            } else {
                $where1[] = $where2[] = 'c.value = ' . $category_ids[0];
            }
        }
        if (!empty($tag)) {
            $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_tags t ON t.entity_id = p.post_id';
            $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_tags t ON t.entity_id = pa.value';
            $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_tags t ON t.entity_id = p.post_id';
            $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_tags t ON t.entity_id = pa.value';
            $where1[] = $where2[] = 't.value = ' . (int)$tag;
        }
        // Filter
        if ($filter) {
            switch ($filter) {
                case 'resolved':
                    $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = p.post_id';
                    $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = pa.value';
                    $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = p.post_id';
                    $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = pa.value';
                    $where1[] = $where2[] = 'r.value = 1';
                    break;
                case 'unresolved':
                    $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = p.post_id';
                    $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = pa.value';
                    $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = p.post_id';
                    $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_resolved r ON r.entity_id = pa.value';
                    $where1[] = $where2[] = 'r.value IS NULL';
                    break;
                case 'closed':
                    $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = p.post_id';
                    $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = pa.value';
                    $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = p.post_id';
                    $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = pa.value';
                    $where1[] = $where2[] = 'cl.value = 1';
                    break;
                case 'open':
                    $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = p.post_id';
                    $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = pa.value';
                    $count_sql1 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = p.post_id';
                    $count_sql2 .= ' LEFT JOIN %1$sentity_field_questions_closed cl ON cl.entity_id = pa.value';
                    $where1[] = $where2[] = 'cl.value IS NULL';
                    break;
            }
        }
        // Sort
        $order = 'DESC';
        switch ($sort) {
            case 'active':
                $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_content_activity a ON a.entity_id = p.post_id';
                $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_content_activity a ON a.entity_id = pa.value';
                $sorter1 = $sorter2 = 'a.active_at';
                break;
            case 'votes':
                $fetch_sql1 .= ' LEFT JOIN %1$sentity_field_voting_updown u ON u.entity_id = p.post_id';
                $fetch_sql2 .= ' LEFT JOIN %1$sentity_field_voting_updown u ON u.entity_id = pa.value';
                $sorter1 = $sorter2 = 'u.sum';
                break;
            case 'oldest':
                $sorter1 = 'p.post_id';
                $sorter2 = 'pa.value';
                $order = 'ASC';
                break;
            default:
                $sorter1 = 'p.post_id';
                $sorter2 = 'pa.value';
                break;
        }
        // Construct SQL
        $where1 = implode(' AND ', $where1);
        $where2 = implode(' AND ', $where2);
        $this->_fetchSql = sprintf('(' . $fetch_sql1 .' WHERE %4$s) UNION ('. $fetch_sql2 .' WHERE %5$s) ORDER BY sorter %6$s', $db->getResourcePrefix(), $sorter1, $sorter2, $where1, $where2, $order);
        $this->_countSql = sprintf('SELECT COUNT(id) FROM ((' . $count_sql1 .' WHERE %2$s) UNION ('. $count_sql2 .' WHERE %3$s)) AS dummy', $db->getResourcePrefix(), $where1, $where2);
        
        return new SabaiFramework_Paginator_Custom(
            array($this, 'countEntities'),
            array($this, 'fetchEntities'),
            $limit,
            array(),
            array($application),
            array()
        );
        
    }
    
    public function countEntities($application)
    {
        return $application->getDB()->query($this->_countSql)->fetchSingle();
    }
    
    public function fetchEntities($application, $limit = 0, $offset = 0)
    {
        $entity_ids = $application->getDB()->query($this->_fetchSql, $limit, $offset)->fetchAllColumns(0);
        return $application->Entity_Entities('content', $entity_ids, true, true); 
    }
}